@@ -88,8 +88,8 @@ class prettyface(object):
             self.children = []
 
         else:  # blender face
-            # self.uv = data.uv
-            self.uv = data.id_data.uv_textures.active.data[data.index].uv  # XXX25
+            uv_layer = data.id_data.uv_layers.active.data
+            self.uv = [uv_layer[i].uv for i in data.loop_indices]
 
             # cos = [v.co for v in data]
             cos = [data.id_data.vertices[v].co for v in data.vertices]  # XXX25
@@ -157,17 +157,17 @@ class prettyface(object):
                 angles_co.sort()
                 I = [i for a, i in angles_co]
 
-                #~ fuv = f.uv
-                fuv = f.id_data.uv_textures.active.data[f.index].uv  # XXX25
+                uv_layer = f.id_data.uv_layers.active.data
+                fuv = [uv_layer[i].uv for i in f.loop_indices]
 
                 if self.rot:
-                    fuv[I[2]] = p1
-                    fuv[I[1]] = p2
-                    fuv[I[0]] = p3
+                    fuv[I[2]][:] = p1
+                    fuv[I[1]][:] = p2
+                    fuv[I[0]][:] = p3
                 else:
-                    fuv[I[2]] = p1
-                    fuv[I[0]] = p2
-                    fuv[I[1]] = p3
+                    fuv[I[2]][:] = p1
+                    fuv[I[0]][:] = p2
+                    fuv[I[1]][:] = p3
 
             f, lens, lensord = uv[0]
 
@@ -178,10 +178,10 @@ class prettyface(object):
                 set_uv(f, (x2, y2), (x2, y1 + margin_h), (x1 + margin_w, y2))
 
         else:  # 1 QUAD
-            uv[1][0], uv[1][1] = x1, y1
-            uv[2][0], uv[2][1] = x1, y2
-            uv[3][0], uv[3][1] = x2, y2
-            uv[0][0], uv[0][1] = x2, y1
+            uv[1][:] = x1, y1
+            uv[2][:] = x1, y2
+            uv[3][:] = x2, y2
+            uv[0][:] = x2, y1
 
     def __hash__(self):
         # None unique hash
@@ -197,12 +197,12 @@ def lightmap_uvpack(meshes,
                       PREF_BOX_DIV=8,
                       PREF_MARGIN_DIV=512
                       ):
-    '''
+    """
     BOX_DIV if the maximum division of the UV map that
     a box may be consolidated into.
     Basically, a lower value will be slower but waist less space
     and a higher value will have more clumpy boxes but more wasted space
-    '''
+    """
     import time
     from math import sqrt
 
@@ -219,15 +219,10 @@ def lightmap_uvpack(meshes,
         face_groups = []
 
     for me in meshes:
-        # Add face UV if it does not exist.
-        # All new faces are selected.
-        if not me.uv_textures:
-            me.uv_textures.new()
-
         if PREF_SEL_ONLY:
-            faces = [f for f in me.faces if f.select]
+            faces = [f for f in me.polygons if f.select]
         else:
-            faces = me.faces[:]
+            faces = me.polygons[:]
 
         if PREF_PACK_IN_ONE:
             face_groups[0].extend(faces)
@@ -237,6 +232,11 @@ def lightmap_uvpack(meshes,
         if PREF_NEW_UVLAYER:
             me.uv_textures.new()
 
+        # Add face UV if it does not exist.
+        # All new faces are selected.
+        if not me.uv_textures:
+            me.uv_textures.new()
+
     for face_sel in face_groups:
         print("\nStarting unwrap")
 
@@ -244,7 +244,7 @@ def lightmap_uvpack(meshes,
             print("\tWarning, less then 4 faces, skipping")
             continue
 
-        pretty_faces = [prettyface(f) for f in face_sel if len(f.vertices) == 4]
+        pretty_faces = [prettyface(f) for f in face_sel if f.loop_total == 4]
 
         # Do we have any triangles?
         if len(pretty_faces) != len(face_sel):
@@ -268,7 +268,7 @@ def lightmap_uvpack(meshes,
 
                 return f, lens, lens_order
 
-            tri_lengths = [trylens(f) for f in face_sel if len(f.vertices) == 3]
+            tri_lengths = [trylens(f) for f in face_sel if f.loop_total == 3]
             del trylens
 
             def trilensdiff(t1, t2):
@@ -528,7 +528,7 @@ def unwrap(operator, context, **kwargs):
         if obj and obj.type == 'MESH':
             meshes = [obj.data]
     else:
-        meshes = list({me for obj in context.selected_objects if obj.type == 'MESH' for me in (obj.data,) if me.faces and me.library is None})
+        meshes = list({me for obj in context.selected_objects if obj.type == 'MESH' for me in (obj.data,) if me.polygons and me.library is None})
 
     if not meshes:
         operator.report({'ERROR'}, "No mesh object")
@@ -545,14 +545,23 @@ from bpy.props import BoolProperty, FloatProperty, IntProperty
 
 
 class LightMapPack(Operator):
-    '''Follow UVs from active quads along continuous face loops'''
+    """Follow UVs from active quads along continuous face loops"""
     bl_idname = "uv.lightmap_pack"
     bl_label = "Lightmap Pack"
-    bl_options = {'REGISTER', 'UNDO'}
+
+    # Disable REGISTER flag for now because this operator might create new
+    # images. This leads to non-proper operator redo because current undo
+    # stack is local for edit mode and can not remove images created by this
+    # oprtator.
+    # Proper solution would be to make undo stack aware of such things,
+    # but for now just disable redo. Keep undo here so unwanted changes to uv
+    # coords might be undone.
+    # This fixes infinite image creation reported there [#30968] (sergey)
+    bl_options = {'UNDO'}
 
     PREF_CONTEXT = bpy.props.EnumProperty(
             name="Selection",
-            items=(('SEL_FACES', "Selected Faces", "Space all UVs evently"),
+            items=(('SEL_FACES', "Selected Faces", "Space all UVs evenly"),
                    ('ALL_FACES', "All Faces", "Average space UVs edge length of each loop"),
                    ('ALL_OBJECTS', "Selected Mesh Object", "Average space UVs edge length of each loop")
                    ),
